package multinomad.tools.kmeans;

import java.util.Iterator;

import multinomad.config.Configuration;
import multinomad.individuals.DoubleIndividual;
import multinomad.individuals.Population;
import multinomad.tools.CommonState;
import multinomad.tools.kmeansML.Centroids;

public class Kmeans implements Iterable<DoubleIndividual>{
	
	private Population centroids;
	private Population rawdata;
	private int k;
	
	public Kmeans(Population poprawdata, int numClusters) {
		centroids = new Population(false);
		k = numClusters;
		rawdata = poprawdata;
		initialize();
	}
	
	private void initialize() {
		if(Configuration.kmeansinit.equals(Configuration.KMEANSRANDOM)) {
			randomInit();
		}else if (Configuration.kmeansinit.equals(Configuration.KMEANSPLUS)) {
			kmeansPlusInit();
		}
	}
	
	private void randomInit() {
		for (int i=0;i<k;i++) {
			
			boolean inserted = false;
			while(!inserted) {
				DoubleIndividual ind = rawdata.getP().get(CommonState.r.nextInt(rawdata.size()));
				if(!centroids.contains(ind)) {
					inserted = true;
					Centroid centroid = new Centroid(ind);
					centroids.add(centroid);
				}
			}
		}
	}
	
	//TODO: Implement the initialization of kmeans++
	private void kmeansPlusInit() {
		
	}
	
	/**
	 * 
	 * @return A populatin of centroids after convergence of kmeans
	 */
	public Population cluster() {
		
		boolean changed = true; // change in at least one cluster assignement
		boolean success = true; // all means computed? (no zero-count cluster)
		int maxCount = rawdata.size()*10; // sanity check
		int ct = 0;
		
		while(changed == true && success == true && ct < maxCount) { // tecnically this is Lloyd's algorithm
			changed = updateClustering(); // (re)assign tuples to clusters
			success = updateMeans(); // Compute new cluster means if possible
			ct++; // k-means typically converges very quickly
		}
		
		return centroids;
	}
	
	// TODO
	private boolean updateClustering() {
		
		boolean changed = false;
		
		for(DoubleIndividual ind : rawdata) {
			double distance = Double.MAX_VALUE;
			int i = 0;
			int index = -1;
			int indexContains = -1;
			for(DoubleIndividual cent : centroids) {
				double distanceToCent =  cent.distance(ind);
				if (distanceToCent < distance) {
					index = i;
					distance = distanceToCent;
				}
				if(((Centroid)cent).isInCluster(ind)) {
					indexContains = i;
				}
				i++;
			}
			
			if (index != indexContains) { // That means that the individual has changed from cluster;
				if (indexContains != -1) {
					((Centroid)centroids.getP().get(indexContains)).removeFromCluster(ind);
				}
				((Centroid)centroids.getP().get(index)).addToCluster(ind);
				changed = true;
			}
		}
		
		return changed;
	}
	
	private boolean updateMeans() {
		if(Configuration.kmeansconvergence.equals(Configuration.KMEANSLLOYD)) {
			return updateMeansLloyd();
		}else if (Configuration.kmeansinit.equals(Configuration.KMEANSBIASED)) {
			return updateMeansBiased();
		}else
			return false;
	}
	
	private boolean updateMeansLloyd() {
		
		for(DoubleIndividual c : centroids) {
			Centroid cent = (Centroid)c;
			if (cent.clusterSize() == 0) return false;
			
			double[] newMean = initVectorToZero(cent.getChr().getLength());
			for(DoubleIndividual itemInCluster : cent) {
				newMean = sumV1toV2(newMean, itemInCluster.getChr().asdouble());
			}
			newMean = divideVectorByValue(newMean, cent.clusterSize());
			cent.updateMean(newMean);
		}
		
		return true;
	}
	
	private double[] initVectorToZero(int length) {
		double[] vector = new double[length];
		for(int i=0;i<vector.length;i++)
			vector[i] = 0;
		return vector;
	}
	
	private double[] sumV1toV2(double[] v1,double[] v2) {
		double[] vector = initVectorToZero(v1.length);
		for(int i=0;i<v1.length;i++)
			vector[i] = v1[i] + v2[i];
		return vector;
	}
	
	private double[] divideVectorByValue(double[] v1,double value) {
		double[] vector = initVectorToZero(v1.length);
		for(int i=0;i<v1.length;i++)
			vector[i] = v1[i] / value;
		return vector;
	}
	
	//TODO
	private boolean updateMeansBiased() {
			
		return false;
	}
	
	public Iterator<DoubleIndividual> iterator() {
		return centroids.iterator();
	}

}
